/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */

#include <glaf.h>
#include <GL/glut.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifndef M_PI
#define M_PI            3.14159265358979323846
#endif

/*Window IDs*/
int	Window[2];

/*Display lists for each window*/
GLuint	SceneList[2];

/*Camera position for each window*/
float	Cam_AngXY[2];
float	Cam_AngXZ[2];
float	Cam_Dist[2];

/*Mouse position for each window*/
int	Mouse_x[2];
int	Mouse_y[2];

/*Menu Selections*/
int	Menu_Sel[2];

/*GLAF contexts IDs for each window*/
GLint	GlafID[2];

/*Image Buffer for glReadPixels/glDrawPixels*/
unsigned char	*ImageBuf;

/*Prototypes*/
int	main(int argc, char** argv);
void	printmessage(char *t);
void	initGL(int window);
void	drawGL(int window);
void	mouseGL(int window, int button, int state, int x, int y);
void	motionGL(int window, int x, int y);
void	menuGL(int window, int sel);
void	display_one(void);
void	display_two(void);
void	mouse_one(int button, int state, int x, int y);
void	mouse_two(int button, int state, int x, int y);
void	motion_one(int x, int y);
void	motion_two(int x, int y);
void	menu_one(int sel);
void	menu_two(int sel);


/********/
/* Main */
/********/
int	main(int argc, char** argv)
	{
	glutInit(&argc, argv);
	
	/*window 1*/
	glutInitWindowSize(400, 400);
	glutInitWindowPosition(0, 10);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	Window[0]=glutCreateWindow("GLAF Convolution -Context 1-");	
	glutDisplayFunc(display_one);
	glutMouseFunc(mouse_one);
	glutMotionFunc(motion_one);
	glutCreateMenu(menu_one);
	glutAddMenuEntry("Disable Convolution", 					0);
	glutAddMenuEntry("Enable Convolution (3x3 Average      -separable-)",		1);
	glutAddMenuEntry("Enable Convolution (5x5 Average      -separable-)",		2);
	glutAddMenuEntry("Enable Convolution (7x7 Average      -separable-)",		3);
	glutAddMenuEntry("Enable Convolution (3x3 North Emboss -separable-)",		4);
	glutAddMenuEntry("Enable Convolution (3x3 Sharpen      -non separable-)",	5);
	glutAddMenuEntry("Enable Convolution (3x3 Edge Detect  -non separable-)",	6);
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	initGL(0);
	
	/*Initialize GLAF context for window 1, in verbose mode*/
	glaf_SetVerbose(printmessage);
	GlafID[0]=glaf_InitializeContext();
	
	/*window 2*/
	glutInitWindowPosition(430, 10);
	Window[1]=glutCreateWindow("GLAF Convolution -Context 2-");	
	glutDisplayFunc(display_two);
	glutMouseFunc(mouse_two);
	glutMotionFunc(motion_two);
	glutCreateMenu(menu_two);
	glutAddMenuEntry("Disable Convolution", 					0);
	glutAddMenuEntry("Enable Convolution (3x3 Average      -separable-)",		1);
	glutAddMenuEntry("Enable Convolution (5x5 Average      -separable-)",		2);
	glutAddMenuEntry("Enable Convolution (7x7 Average      -separable-)",		3);
	glutAddMenuEntry("Enable Convolution (3x3 North Emboss -separable-)",		4);
	glutAddMenuEntry("Enable Convolution (3x3 Sharpen      -non separable-)",	5);
	glutAddMenuEntry("Enable Convolution (3x3 Edge Detect  -non separable-)",	6);
	glutAttachMenu(GLUT_RIGHT_BUTTON);
	initGL(1);
	
	/*Buffer for glReadPixels/glDrawPixels (max size 2048x2048) */
	ImageBuf=(unsigned char *)malloc(glaf_ImageSize(2048,2048,GL_RGBA,GL_UNSIGNED_BYTE,4));
	
	/*Initialize GLAF context for window 2, in non-verbose mode*/
	glaf_SetVerbose(NULL);
	GlafID[1]=glaf_InitializeContext();
	
	/*main loop*/
	glutMainLoop();
	return 0;
	}
	
	
/***********************/
/*GLAF verbose callback*/
/***********************/
void	printmessage(char *t)
	{
	printf("\n%s\n",t);
	fflush(stdout);
	}
	

/******************************/
/*Window-independent functions*/
/******************************/
void	initGL(int window)
	{
	Cam_AngXY[window]=0.0;
	Cam_AngXZ[window]=22.5;
	Cam_Dist[window]=5.0;
	
	Menu_Sel[window]=1; /*By default, enable convolution*/
	
	SceneList[window]=glGenLists(1);
	glNewList(SceneList[window],GL_COMPILE);
	glutSolidDodecahedron();
	glEndList();
	
	glClearColor(0.5,0.5,1.0,0.0);
	glClearDepth(1.0);
	
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glEnable(GL_CULL_FACE);
	}
	
	
void	drawGL(int window)
	{
	int		w,h,f,rw,rh;
	GLfloat		p[4];
	GLfloat		light_amb[] = { 0.0, 0.0, 0.0, 1.0 };
	GLfloat		light_dif[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat		yellow[] =    { 1.0, 1.0, 0.0, 1.0 };
	GLfloat		row[7];
	GLfloat		column[7];
	GLfloat		filt3[3][3];
	GLfloat		weight;
	
	/*Make current the GLAF context of the current window*/
	glaf_ReadyForContext(GlafID[window]);
	
	/*Projection matrix*/
	w=glutGet(GLUT_WINDOW_WIDTH);
	h=glutGet(GLUT_WINDOW_HEIGHT);
	glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
	gluPerspective(45.0,(GLdouble)w/(GLdouble)h,0.05,100.0);
        
	/*Framebuffer clear (we clear buffers just before CPU trigonometry work in purpose) */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	p[0]=Cam_Dist[window]*cos(Cam_AngXY[window]*M_PI/180.0)*cos(Cam_AngXZ[window]*M_PI/180.0);
	p[1]=Cam_Dist[window]*sin(Cam_AngXY[window]*M_PI/180.0)*cos(Cam_AngXZ[window]*M_PI/180.0);
	p[2]=Cam_Dist[window]*sin(Cam_AngXZ[window]*M_PI/180.0);
	p[3]=1.0;
	
	/*Modelview Matrix*/
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();
     	gluLookAt(p[0], p[1], p[2], 0.0, 0.0, 0.0, 0.0, 0.0, 1.0);
	
	/*Light*/
	glLightfv (GL_LIGHT0, GL_POSITION, p);
	glLightfv (GL_LIGHT0, GL_AMBIENT, light_amb);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, light_dif);
	glEnable (GL_LIGHT0);
	glEnable (GL_LIGHTING);
	
	/*Yellow Material*/	
	glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, yellow);
        
	/*Draw scene */
	glCallList(SceneList[window]);
		
	glDisable (GL_LIGHT0);
	glDisable (GL_LIGHTING);
	
	/*Convolution*/
	if(Menu_Sel[window]!=0)
		{
		/*convolution enabled*/
		if(glaf_IsConvolutionSupported())
			{
			glPixelStorei(GL_UNPACK_SWAP_BYTES, 0);
			glPixelStorei(GL_UNPACK_LSB_FIRST, 0);
			glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
			glPixelStorei(GL_UNPACK_SKIP_ROWS, 0);
			glPixelStorei(GL_UNPACK_SKIP_PIXELS, 0);
			glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
			
			glPixelStorei(GL_PACK_SWAP_BYTES, 0);
			glPixelStorei(GL_PACK_LSB_FIRST, 0);
			glPixelStorei(GL_PACK_ROW_LENGTH, 0);
			glPixelStorei(GL_PACK_SKIP_ROWS, 0);
			glPixelStorei(GL_PACK_SKIP_PIXELS, 0);
			glPixelStorei(GL_PACK_ALIGNMENT, 4);
			
			/*Image Buffer. Read Pixels without convolution*/
			glDisable(GLAF_CONVOLUTION_2D);
			glDisable(GLAF_SEPARABLE_2D);
			rw=w;
			rh=h;
			if(rw>2048)
				{
				/*Wow! ... what a display mode!*/
				rw=2048;
				}
			if(rh>2048)
				{
				/*Wow! ... what a display mode!*/
				rh=2048;
				}
			glReadPixels(0,0,rw,rh,GL_RGBA,GL_UNSIGNED_BYTE,ImageBuf);
			
			/*Set convolution kernel and enable convolution*/
			glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
			switch(Menu_Sel[window])
				{
				case 1:
					/*Average 3x3*/
					weight=1.0/(3.0);
	  				for(f=0;f<3;f++)
						{
						row[f]=weight;
						column[f]=weight;
						}
					glaf_SeparableFilter2D(GLAF_SEPARABLE_2D, GL_LUMINANCE, 3, 3, GL_LUMINANCE, GL_FLOAT, row, column);
					glDisable(GLAF_CONVOLUTION_2D);
					glEnable(GLAF_SEPARABLE_2D);
					break;
				case 2:
					/*Average 5x5*/
					weight=1.0/(5.0);
	  				for(f=0;f<5;f++)
						{
						row[f]=weight;
						column[f]=weight;
						}
					glaf_SeparableFilter2D(GLAF_SEPARABLE_2D, GL_LUMINANCE, 5, 5, GL_LUMINANCE, GL_FLOAT, row, column);
					glDisable(GLAF_CONVOLUTION_2D);
					glEnable(GLAF_SEPARABLE_2D);
					break;
				case 3:
					/*Average 7x7*/
					weight=1.0/(7.0);
	  				for(f=0;f<7;f++)
						{
						row[f]=weight;
						column[f]=weight;
						}
					glaf_SeparableFilter2D(GLAF_SEPARABLE_2D, GL_LUMINANCE, 7, 7, GL_LUMINANCE, GL_FLOAT, row, column);
					glDisable(GLAF_CONVOLUTION_2D);
					glEnable(GLAF_SEPARABLE_2D);
					break;
				case 4:
					/*North Emboss 3x3*/
					column[2]=-1;
					column[1]=.25;
					column[0]=1;
					row[0]=1;
					row[1]=2;
					row[2]=1;
					glaf_SeparableFilter2D(GLAF_SEPARABLE_2D, GL_LUMINANCE, 3, 3, GL_LUMINANCE, GL_FLOAT, row, column);
					glDisable(GLAF_CONVOLUTION_2D);
					glEnable(GLAF_SEPARABLE_2D);
					break;
				case 5:
					/*Sharpen 3x3*/
					filt3[2][0]=-1;
					filt3[2][1]=-.5;
					filt3[2][2]=-1;
					
					filt3[1][0]=-.5;
					filt3[1][1]=7;
					filt3[1][2]=-.5;
					
					filt3[0][0]=-1;
					filt3[0][1]=-.5;
					filt3[0][2]=-1;
					
					glaf_ConvolutionFilter2D(GLAF_CONVOLUTION_2D, GL_LUMINANCE, 3, 3, GL_LUMINANCE, GL_FLOAT, filt3);
					glDisable(GLAF_SEPARABLE_2D);
					glEnable(GLAF_CONVOLUTION_2D);
					break;
				case 6:
					/*Edge Detect 3x3*/
					filt3[2][0]=-1;
					filt3[2][1]=-1;
					filt3[2][2]=-1;
					
					filt3[1][0]=-1;
					filt3[1][1]=8;
					filt3[1][2]=-1;
			
					filt3[0][0]=-1;
					filt3[0][1]=-1;
					filt3[0][2]=-1;
					
					glaf_ConvolutionFilter2D(GLAF_CONVOLUTION_2D, GL_LUMINANCE, 3, 3, GL_LUMINANCE, GL_FLOAT, filt3);
					glDisable(GLAF_SEPARABLE_2D);
					glEnable(GLAF_CONVOLUTION_2D);
					break;
				default:
					break;
				}
			
			/*Perform convolution: Draw pixels */
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			glOrtho(0,1,0,1,0,1); 
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
			glRasterPos2f(0.0,0.0);
			glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
			glDrawPixels(rw,rh,GL_RGBA,GL_UNSIGNED_BYTE,ImageBuf);
				
			glDisable(GLAF_SEPARABLE_2D);
			glDisable(GLAF_CONVOLUTION_2D);
			}
		else
			{
			printmessage("Warning: GLAF convolution feature unsupported!");
			}
		}
	
	/*Swap Buffers */
	glutSwapBuffers();	
	}
	
	
void	mouseGL(int window, int button, int state, int x, int y)
	{
	Mouse_x[window]=x;
	Mouse_y[window]=y;
	}
	
	
void	motionGL(int window, int x, int y)
	{
	Cam_AngXY[window]+=(float)(Mouse_x[window]-x)*.5;
	Cam_AngXZ[window]+=(float)(y-Mouse_y[window])*.5;
	if(Cam_AngXZ[window]>89.0)
		{
		Cam_AngXZ[window]=89.0;
		}
	if(Cam_AngXZ[window]<-89.0)
		{
		Cam_AngXZ[window]=-89.0;
		}
	Mouse_x[window]=x;
	Mouse_y[window]=y;
	
	/*redraw*/
	glutPostRedisplay();
	}
	
	
void	menuGL(int window, int sel)
	{
	Menu_Sel[window]=sel;
	glutPostRedisplay();
	}
	
	
/*****************************************************************/
/*GLUT callbacks -> They redirect to window-independent functions*/
/*****************************************************************/
void	display_one(void)
	{
	drawGL(0);
	}
	
	
void	display_two(void)
	{
	drawGL(1);
	}
	
	
void	mouse_one(int button, int state, int x, int y)
	{
	mouseGL(0,button,state,x,y);
	}
	
	
void	mouse_two(int button, int state, int x, int y)
	{
	mouseGL(1,button,state,x,y);
	}
	
	
void	motion_one(int x, int y)
	{
	motionGL(0,x,y);
	}
	
	
void	motion_two(int x, int y)
	{
	motionGL(1,x,y);
	}
	
	
void	menu_one(int sel)
	{
	menuGL(0,sel);
	}
	
	
void	menu_two(int sel)
	{
	menuGL(1,sel);
	}
	
	
