/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "private.h"
#include "blending_extensionsP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextBlending(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	GLboolean	arb_imaging=GL_FALSE;
	GLboolean	old_blend_equation=GL_FALSE;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	
	/*Cache ARB_imaging availability*/
	if((major>1)||(minor>1))
		{
		/*ARB_imaging requires OpenGL 1.2 or greater*/
		arb_imaging=glaf_IsOpenGLExtensionSupported("GL_ARB_imaging");
		}
	
	/*Constant Color Blending*/
	ctx->glBlendColorPtr=NULL;
	if(arb_imaging)
		{
		/*Let's try to use GL_ARB_imaging*/
		ctx->glBlendColorPtr=(glBlendColorProc)_glaf_LookupFunction("glBlendColor");
		if(ctx->glBlendColorPtr!=NULL)
			{
			_glaf_Verbose("Constant Color Blending: Using GL_ARB_imaging.");
			}
		}
	if(ctx->glBlendColorPtr==NULL)
		{
		/*Let's try to use GL_EXT_blend_color*/
		if(glaf_IsOpenGLExtensionSupported("GL_EXT_blend_color"))
			{
			ctx->glBlendColorPtr=(glBlendColorProc)_glaf_LookupFunction("glBlendColorEXT");
			if(ctx->glBlendColorPtr!=NULL)
				{
				_glaf_Verbose("Constant Color Blending: Using GL_EXT_blend_color.");
				}
			}
		}
	if(ctx->glBlendColorPtr==NULL)
		{
		_glaf_Verbose("Constant Color Blending: Not Supported.");
		}
		
	/*Equation Blending Extensions: MinMax, LogicOp, Subtract*/
	ctx->glBlendEquationPtr=NULL;
	ctx->IsBlendMinMaxSupported=GL_FALSE;
	ctx->IsBlendSubtractSupported=GL_FALSE;
	if(arb_imaging)
		{
		/*Let's try to use GL_ARB_imaging. IMPORTANT: GL_ARB_imaging doesn't include GL_EXT_blend_logic_op */
		ctx->glBlendEquationPtr=(glBlendEquationProc)_glaf_LookupFunction("glBlendEquation");
		if(ctx->glBlendEquationPtr!=NULL)
			{
			_glaf_Verbose("MinMax Blending and Blend Subtract: Using GL_ARB_imaging.");
			ctx->IsBlendMinMaxSupported=GL_TRUE;
			ctx->IsBlendSubtractSupported=GL_TRUE;
			}
		}
	if(ctx->glBlendEquationPtr==NULL)
		{
		/*Let's try to use older blending extensions*/
		if(glaf_IsOpenGLExtensionSupported("GL_EXT_blend_minmax"))
			{
			_glaf_Verbose("MinMax Blending: Using GL_EXT_blend_minmax.");
			old_blend_equation=GL_TRUE;
			ctx->IsBlendMinMaxSupported=GL_TRUE;
			}
		else
			{
			_glaf_Verbose("MinMax Blending: Not Supported.");
			}
		if(glaf_IsOpenGLExtensionSupported("GL_EXT_blend_subtract"))
			{
			_glaf_Verbose("Blend Subtract: Using GL_EXT_blend_subtract.");
			old_blend_equation=GL_TRUE;
			ctx->IsBlendSubtractSupported=GL_TRUE;
			}
		else
			{
			_glaf_Verbose("Blend Subtract: Not Supported.");
			}
		if((major==1)&&(minor==0))
			{
			/* OpenGL 1.0:  RGBA LogicOp NOT supported by Core*/
			if(glaf_IsOpenGLExtensionSupported("GL_EXT_blend_logic_op"))
				{
				_glaf_Verbose("RGBA LogicOp: Using GL_EXT_blend_logic_op.");
				old_blend_equation=GL_TRUE;
				ctx->_glaf_EnableRGBALogicOpPtr=_glaf_EnableRGBALogicOpExt;
				ctx->_glaf_DisableRGBALogicOpPtr=_glaf_DisableRGBALogicOpExt;
				}
			else
				{
				_glaf_Verbose("RGBA LogicOp: Not Supported.");
				ctx->_glaf_EnableRGBALogicOpPtr=NULL;
				ctx->_glaf_DisableRGBALogicOpPtr=NULL;
				}
			}
		if(old_blend_equation)
			{
			ctx->glBlendEquationPtr=(glBlendEquationProc)_glaf_LookupFunction("glBlendEquationEXT");
			if(ctx->glBlendEquationPtr==NULL)
				{
				/* Ooops!! Some extensions that require BlendEquation are supported, 
				   but BlendEquation is not. Nonsense!! (this error should never happen) */
				_glaf_Verbose("Unexpected situation: glBlendEquationEXT not supported: MinMax and Subtract Blending Extensions are NOT supported, although the implementation claims to support some of them.");
				ctx->IsBlendMinMaxSupported=GL_FALSE;
				ctx->IsBlendSubtractSupported=GL_FALSE;
				if((major==1)&&(minor==0))
					{
					/*OpenGL 1.0*/
					if(ctx->_glaf_EnableRGBALogicOpPtr!=NULL)
						{
						_glaf_Verbose("Unexpected situation: glBlendEquationEXT not supported: RGBA LogicOp NOT supported, although the implementation claims to support it.");
						ctx->_glaf_EnableRGBALogicOpPtr=NULL;
						ctx->_glaf_DisableRGBALogicOpPtr=NULL;
						}
					}
				}
			}
		}
	if((major>1)||(minor>0))
		{
		/* OpenGL 1.1 or greater: RGBA LogicOp supported by Core*/
		_glaf_Verbose("RGBA LogicOp: Supported by OpenGL Core.");
		ctx->_glaf_EnableRGBALogicOpPtr=_glaf_EnableRGBALogicOpCore;
		ctx->_glaf_DisableRGBALogicOpPtr=_glaf_DisableRGBALogicOpCore;
		}
	}

	
void	_glaf_EnableRGBALogicOpExt(void)
	{
	glEnable(GL_BLEND);
	GLB_current->glBlendEquationPtr(GL_LOGIC_OP);
	}
	
	
void	_glaf_DisableRGBALogicOpExt(void)
	{
	glDisable(GL_BLEND);
	GLB_current->glBlendEquationPtr(GLAF_FUNC_ADD);
	}
	

void	_glaf_EnableRGBALogicOpCore(void)
	{
	glEnable(GLAF_COLOR_LOGIC_OP);
	}
	
	
void	_glaf_DisableRGBALogicOpCore(void)
	{
	glDisable(GLAF_COLOR_LOGIC_OP);
	glDisable(GL_BLEND);
	if(GLB_current->glBlendEquationPtr!=NULL)
		{
		GLB_current->glBlendEquationPtr(GLAF_FUNC_ADD);
		}
	}
	
	
