/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */

#ifdef _WIN32
#include <windows.h>
#endif

#include "glincludes.h"
#include "glaf.h"
#include "capabilities.h"

#include <string.h>
#include <stdio.h>

#if defined( GLAF_GLX )
#include <dlfcn.h>
#endif


void	glaf_Version(GLint *major, GLint *minor, GLint *release)
	{
	/*Returns the GLAF major, minor, and release version numbers.
	*/
	
	(*major)=	1;
	(*minor)=	0;
	(*release)=	0;
	return;
	}


void	glaf_OpenGLVersion(GLint *major, GLint *minor, GLint *release)
	{
	/*Returns the OpenGL major, minor, and release version numbers. Examples: OpenGL 1.0 has 1,0,0. OpenGL 1.2.1 has 1,2,1.
	*/
	char		*p;
	
	p=(char *)glGetString(GL_VERSION);
	sscanf(p,"%d.%d.%d",major,minor,release);
	return;
	}


GLboolean	glaf_IsOpenGLExtensionSupported(char *extName)
	{
	/*Returns GL_TRUE if the OpenGL Extension whose name is *extName is supported by the system, or GL_FALSE otherwise.
	
	  The *extName string must follow the OpenGL extensions naming scheme (ie: "GL_type_extension", like GL_EXT_convolution)
	*/
	
	return _glaf_SearchExtensionsString((char *)glGetString(GL_EXTENSIONS), extName);
	}
	
	
/************************************************/
/*Private functions (for internal GLAF use only)*/
/************************************************/
GLboolean	_glaf_SearchExtensionsString(char *extString, char *extName)
	{
	/*Returns GL_TRUE if the *extName string appears in the *extString string, surrounded by white spaces, or GL_FALSE otherwise.
	*/
	char		*p;
	char 		*end;
	int		extNameLen;
	int		n;
	
	extNameLen = strlen(extName);
	
	p=extString;
	end = p + strlen(p);
	
	while (p < end)
		{
		n = strcspn(p, " ");
		if ((extNameLen == n) && (strncmp(extName, p, n) == 0))
			{
			return GL_TRUE;
			}
		p += (n + 1);
		}
	return GL_FALSE;
	}
	

void	(*_glaf_LookupFunction(const char *func))()
	{
	
#if defined( GLAF_WGL )
	return (void (*)()) wglGetProcAddress(func);
#endif
#if defined( GLAF_GLX )
  #if defined( GLAF_GLX_NO_PROC_ADDRESS )
	void *libHandle;
	void (*fptr)();
	libHandle = dlopen("libGL.so", RTLD_LAZY);
	fptr = (void (*)()) dlsym(libHandle, func);
	dlclose(libHandle);
	return fptr;
  #else
	return glXGetProcAddressARB(func);
  #endif
#endif
	}
	
	
	
