/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "private.h"
#include "color_subtableP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextColorSubTable(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>1))
		{
		/*ARB_imaging requires OpenGL 1.2 or greater*/
		if(glaf_IsOpenGLExtensionSupported("GL_ARB_imaging"))
			{
			/* ARB_imaging supported: We try to use it */
			_glaf_FillContextColorSubTableARBImaging(ctx);
			if(_glaf_TestContextColorSubTable(ctx))
				{
				/*Success!!*/
				_glaf_Verbose("Color SubTable: Using GL_ARB_imaging.");
				return;
				}
			}
		}
	/*Let's see if we can use GL_EXT_color_subtable*/
	if(glaf_IsOpenGLExtensionSupported("GL_EXT_color_subtable"))
		{
		/*It seems we can use GL_EXT_color_subtable*/
		_glaf_FillContextColorSubTableExt(ctx);
		if(_glaf_TestContextColorSubTable(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Color SubTable: Using GL_EXT_color_subtable.");
			return;
			}
		}
	
	/*Nope, Color SubTable is not supported*/
	_glaf_Verbose("Color SubTable: Not Supported.");
	ctx->glColorSubTablePtr=NULL;
	ctx->glCopyColorSubTablePtr=NULL;
	}

	
void	_glaf_FillContextColorSubTableARBImaging(struct GLAFST_context *ctx)
	{
	ctx->glColorSubTablePtr=(glColorSubTableProc)_glaf_LookupFunction("glColorSubTable");
	ctx->glCopyColorSubTablePtr=(glCopyColorSubTableProc)_glaf_LookupFunction("glCopyColorSubTable");
	}
	
	
void	_glaf_FillContextColorSubTableExt(struct GLAFST_context *ctx)
	{
	ctx->glColorSubTablePtr=(glColorSubTableProc)_glaf_LookupFunction("glColorSubTableEXT");
	ctx->glCopyColorSubTablePtr=(glCopyColorSubTableProc)_glaf_LookupFunction("glCopyColorSubTableEXT");
	}
	
	
GLboolean	_glaf_TestContextColorSubTable(struct GLAFST_context *ctx)
	{
	if(ctx->glColorSubTablePtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyColorSubTablePtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	


