/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "public.h"


GLboolean	glaf_IsColorTableSupported(void)
	{
	/*It returns GL_TRUE if color table is supported, or GL_FALSE otherwise.
	
	  The use of color table is exactly the same as the GL_ARB_imaging color table extension, or the GL_SGI_color_table extension.
	  
	  Enumerants follow the GLAF syntax:
						GLAF_COLOR_TABLE				
						GLAF_POST_CONVOLUTION_COLOR_TABLE		
						GLAF_POST_COLOR_MATRIX_COLOR_TABLE		
						GLAF_PROXY_COLOR_TABLE				
						GLAF_PROXY_POST_CONVOLUTION_COLOR_TABLE		
						GLAF_PROXY_POST_COLOR_MATRIX_COLOR_TABLE	
						GLAF_COLOR_TABLE_SCALE				
						GLAF_COLOR_TABLE_BIAS				
						GLAF_COLOR_TABLE_FORMAT				
						GLAF_COLOR_TABLE_WIDTH				
						GLAF_COLOR_TABLE_RED_SIZE			
						GLAF_COLOR_TABLE_GREEN_SIZE			
						GLAF_COLOR_TABLE_BLUE_SIZE			
						GLAF_COLOR_TABLE_ALPHA_SIZE			
						GLAF_COLOR_TABLE_LUMINANCE_SIZE			
						GLAF_COLOR_TABLE_INTENSITY_SIZE			

	
	  GLAF functions for the color table feature (use the same as GL_ARB_imaging or GL_SGI_color_table):
	
						glaf_ColorTable
						glaf_ColorTableParameteriv
						glaf_ColorTableParameterfv
						glaf_CopyColorTable
						glaf_GetColorTable
						glaf_GetColorTableParameterfv
						glaf_GetColorTableParameteriv

	
	  Note that it's very important to call glaf_IsColorTableSupported *before* you use any of the 
	  above functions. Otherwise, if you use them and they're not supported, the application is 
	  likely to crash.
	*/
	if(GLB_current->glColorTablePtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}


void	glaf_ColorTable( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *table )
	{
	GLB_current->glColorTablePtr( target, internalformat, width, format, type, table );
	}
	
	
void	glaf_ColorTableParameteriv(GLenum target, GLenum pname, const GLint *params)
	{
	GLB_current->glColorTableParameterivPtr( target, pname, params);
	}
	
	
void	glaf_ColorTableParameterfv(GLenum target, GLenum pname, const GLfloat *params)
	{
	GLB_current->glColorTableParameterfvPtr( target, pname, params);
	}
	
	
void	glaf_CopyColorTable( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width )
	{
	GLB_current->glCopyColorTablePtr( target, internalformat, x, y, width );
	}
	
	
void	glaf_GetColorTable( GLenum target, GLenum format, GLenum type, GLvoid *table )
	{
	GLB_current->glGetColorTablePtr( target, format, type, table );
	}
	
	
void	glaf_GetColorTableParameterfv( GLenum target, GLenum pname, GLfloat *params )
	{
	GLB_current->glGetColorTableParameterfvPtr( target, pname, params );
	}
	
	
void	glaf_GetColorTableParameteriv( GLenum target, GLenum pname, GLint *params )
	{
	GLB_current->glGetColorTableParameterivPtr( target, pname, params );
	}
	
