/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "private.h"
#include "color_tableP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextColorTable(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>1))
		{
		/*ARB_imaging requires OpenGL 1.2 or greater*/
		if(glaf_IsOpenGLExtensionSupported("GL_ARB_imaging"))
			{
			/* ARB_imaging supported: We try to use it */
			_glaf_FillContextColorTableARBImaging(ctx);
			if(_glaf_TestContextColorTable(ctx))
				{
				/*Success!!*/
				_glaf_Verbose("Color Table: Using GL_ARB_imaging.");
				return;
				}
			}
		}
	/*Let's see if we can use GL_SGI_color_table*/
	if(glaf_IsOpenGLExtensionSupported("GL_SGI_color_table"))
		{
		/*It seems we can use GL_SGI_color_table*/
		_glaf_FillContextColorTableSgi(ctx);
		if(_glaf_TestContextColorTable(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Color Table: Using GL_SGI_color_table.");
			return;
			}
		}
	
	/*Nope, Color Table is not supported*/
	_glaf_Verbose("Color Table: Not Supported.");
	ctx->glColorTablePtr=NULL;
	ctx->glColorTableParameterivPtr=NULL;
	ctx->glColorTableParameterfvPtr=NULL;
	ctx->glCopyColorTablePtr=NULL;
	ctx->glGetColorTablePtr=NULL;
	ctx->glGetColorTableParameterfvPtr=NULL;
	ctx->glGetColorTableParameterivPtr=NULL;
	}

	
void	_glaf_FillContextColorTableARBImaging(struct GLAFST_context *ctx)
	{
	ctx->glColorTablePtr=(glColorTableProc)_glaf_LookupFunction("glColorTable");
	ctx->glColorTableParameterivPtr=(glColorTableParameterivProc)_glaf_LookupFunction("glColorTableParameteriv");
	ctx->glColorTableParameterfvPtr=(glColorTableParameterfvProc)_glaf_LookupFunction("glColorTableParameterfv");
	ctx->glCopyColorTablePtr=(glCopyColorTableProc)_glaf_LookupFunction("glCopyColorTable");
	ctx->glGetColorTablePtr=(glGetColorTableProc)_glaf_LookupFunction("glGetColorTable");
	ctx->glGetColorTableParameterfvPtr=(glGetColorTableParameterfvProc)_glaf_LookupFunction("glGetColorTableParameterfv");
	ctx->glGetColorTableParameterivPtr=(glGetColorTableParameterivProc)_glaf_LookupFunction("glGetColorTableParameteriv");
	}
	
	
void	_glaf_FillContextColorTableSgi(struct GLAFST_context *ctx)
	{
	ctx->glColorTablePtr=(glColorTableProc)_glaf_LookupFunction("glColorTableSGI");
	ctx->glColorTableParameterivPtr=(glColorTableParameterivProc)_glaf_LookupFunction("glColorTableParameterivSGI");
	ctx->glColorTableParameterfvPtr=(glColorTableParameterfvProc)_glaf_LookupFunction("glColorTableParameterfvSGI");
	ctx->glCopyColorTablePtr=(glCopyColorTableProc)_glaf_LookupFunction("glCopyColorTableSGI");
	ctx->glGetColorTablePtr=(glGetColorTableProc)_glaf_LookupFunction("glGetColorTableSGI");
	ctx->glGetColorTableParameterfvPtr=(glGetColorTableParameterfvProc)_glaf_LookupFunction("glGetColorTableParameterfvSGI");
	ctx->glGetColorTableParameterivPtr=(glGetColorTableParameterivProc)_glaf_LookupFunction("glGetColorTableParameterivSGI");
	}
	
	
GLboolean	_glaf_TestContextColorTable(struct GLAFST_context *ctx)
	{
	if(ctx->glColorTablePtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glColorTableParameterivPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glColorTableParameterfvPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyColorTablePtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetColorTablePtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetColorTableParameterfvPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetColorTableParameterivPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
