/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */

#ifndef __contexts_h__
#define __contexts_h__

#ifndef _WIN32
#define APIENTRY
#endif

/* Typedef's for pointers to generic GLAF void-void functions */
typedef void		(*glaf_VoidVoidProc)(void);

/* Typedef's for pointers to Texture Objects Management functions */
typedef void		(APIENTRY *glGenTexturesProc)(GLsizei n, GLuint *textures);
typedef void 		(APIENTRY *glDeleteTexturesProc)(GLsizei n, const GLuint *textures);
typedef GLboolean	(APIENTRY *glIsTextureProc)(GLuint texture);
typedef void		(APIENTRY *glBindTextureProc)(GLenum target, GLuint texture);
typedef GLboolean	(APIENTRY *glAreTexturesResidentProc)(GLsizei n, const GLuint *textures, GLboolean *residences);
typedef void 		(APIENTRY *glPrioritizeTexturesProc)(GLsizei n, const GLuint *textures, const GLclampf *priorities);

/* Typedef's for pointers to SubTexture functions */
typedef void		(APIENTRY *glTexSubImage1DProc)( GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, const GLvoid *pixels );
typedef void		(APIENTRY *glTexSubImage2DProc)( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels );

/* Typedef's for pointers to Copy Texture functions */
typedef void		(APIENTRY *glCopyTexImage1DProc)( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border );
typedef void		(APIENTRY *glCopyTexImage2DProc)( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border );
typedef void		(APIENTRY *glCopyTexSubImage1DProc)( GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width );
typedef void		(APIENTRY *glCopyTexSubImage2DProc)( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height );

/* Typedef's for pointers to Blending functions */
typedef void		(APIENTRY *glBlendColorProc)(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
typedef void		(APIENTRY *glBlendEquationProc)(GLenum mode);

/* Typedef's for pointers to Polygon Offset functions */
typedef void		(APIENTRY *glPolygonOffsetProc)( GLfloat factor, GLfloat units );

/* Typedef's for pointers to Convolution functions */
typedef void		(APIENTRY *glConvolutionFilter1DProc)( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *image );
typedef void		(APIENTRY *glConvolutionFilter2DProc)( GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *image );
typedef void		(APIENTRY *glConvolutionParameterfProc)( GLenum target, GLenum pname, GLfloat params );
typedef void		(APIENTRY *glConvolutionParameterfvProc)( GLenum target, GLenum pname, const GLfloat *params );
typedef void		(APIENTRY *glConvolutionParameteriProc)( GLenum target, GLenum pname, GLint params );
typedef void		(APIENTRY *glConvolutionParameterivProc)( GLenum target, GLenum pname, GLint *params );
typedef void		(APIENTRY *glCopyConvolutionFilter1DProc)( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width );
typedef void		(APIENTRY *glCopyConvolutionFilter2DProc)( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height);
typedef void		(APIENTRY *glGetConvolutionFilterProc)( GLenum target, GLenum format, GLenum type, GLvoid *image );
typedef void		(APIENTRY *glGetConvolutionParameterfvProc)( GLenum target, GLenum pname, GLfloat *params );
typedef void		(APIENTRY *glGetConvolutionParameterivProc)( GLenum target, GLenum pname, GLint *params );
typedef void		(APIENTRY *glSeparableFilter2DProc)( GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *row, const GLvoid *column );
typedef void		(APIENTRY *glGetSeparableFilterProc)( GLenum target, GLenum format, GLenum type, GLvoid *row, GLvoid *column, GLvoid *span );

/* Typedef's for pointers to Histogram functions */
typedef void		(APIENTRY *glHistogramProc)( GLenum target, GLsizei width, GLenum internalformat, GLboolean sink );
typedef void		(APIENTRY *glResetHistogramProc)( GLenum target );
typedef void		(APIENTRY *glGetHistogramProc)( GLenum target, GLboolean reset, GLenum format, GLenum type, GLvoid *values );
typedef void		(APIENTRY *glGetHistogramParameterfvProc)( GLenum target, GLenum pname, GLfloat *params );
typedef void		(APIENTRY *glGetHistogramParameterivProc)( GLenum target, GLenum pname, GLint *params );
typedef void		(APIENTRY *glMinmaxProc)( GLenum target, GLenum internalformat, GLboolean sink );
typedef void		(APIENTRY *glResetMinmaxProc)( GLenum target );
typedef void		(APIENTRY *glGetMinmaxProc)( GLenum target, GLboolean reset, GLenum format, GLenum types, GLvoid *values );
typedef void		(APIENTRY *glGetMinmaxParameterfvProc)( GLenum target, GLenum pname, GLfloat *params );
typedef void		(APIENTRY *glGetMinmaxParameterivProc)( GLenum target, GLenum pname, GLint *params );

/* Typedef's for pointers to Color Table functions */
typedef void		(APIENTRY *glColorTableProc)( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *table );
typedef void		(APIENTRY *glColorTableParameterivProc)(GLenum target, GLenum pname, const GLint *params);
typedef void		(APIENTRY *glColorTableParameterfvProc)(GLenum target, GLenum pname, const GLfloat *params);
typedef void		(APIENTRY *glCopyColorTableProc)( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width );
typedef void		(APIENTRY *glGetColorTableProc)( GLenum target, GLenum format, GLenum type, GLvoid *table );
typedef void		(APIENTRY *glGetColorTableParameterfvProc)( GLenum target, GLenum pname, GLfloat *params );
typedef void		(APIENTRY *glGetColorTableParameterivProc)( GLenum target, GLenum pname, GLint *params );

/* Typedef's for pointers to Color SubTable functions */
typedef void		(APIENTRY *glColorSubTableProc)( GLenum target, GLsizei start, GLsizei count, GLenum format, GLenum type, const GLvoid *data );
typedef void		(APIENTRY *glCopyColorSubTableProc)( GLenum target, GLsizei start, GLint x, GLint y, GLsizei width );


/* OpenGL Context data structure */
struct	GLAFST_context
	{
	/* 'In use' flag (for allocating, freeing, and reallocating contexts) */
	GLboolean			IsBeingUsed;
	
	/* Availability of 'Separate Specular Color' feature */
	GLboolean			IsSeparateSpecularColorSupported;
	
	/* Availability of 'Texture Formats' feature */
	GLboolean			IsTextureFormatsSupported;
	
	/* Pointers to Texture Objects Management functions */
	glGenTexturesProc		glGenTexturesPtr;
	glDeleteTexturesProc		glDeleteTexturesPtr;
	glIsTextureProc			glIsTexturePtr;		
	glBindTextureProc		glBindTexturePtr;		
	glAreTexturesResidentProc	glAreTexturesResidentPtr;
	glPrioritizeTexturesProc	glPrioritizeTexturesPtr;
	
	/* Pointers to SubTexture functions */
	glTexSubImage1DProc		glTexSubImage1DPtr;
	glTexSubImage2DProc		glTexSubImage2DPtr;

	/* Pointers to Copy Texture functions */
	glCopyTexImage1DProc		glCopyTexImage1DPtr;
	glCopyTexImage2DProc		glCopyTexImage2DPtr;
	glCopyTexSubImage1DProc		glCopyTexSubImage1DPtr;
	glCopyTexSubImage2DProc		glCopyTexSubImage2DPtr;

	/* Pointers to Blending functions */
	glBlendColorProc		glBlendColorPtr;
	glBlendEquationProc		glBlendEquationPtr;
	GLboolean			IsBlendMinMaxSupported;
	GLboolean			IsBlendSubtractSupported;
	
	/* Pointers to LogicOp functions */
	glaf_VoidVoidProc		_glaf_EnableRGBALogicOpPtr;
	glaf_VoidVoidProc		_glaf_DisableRGBALogicOpPtr;
	
	/* Pointers to PolygonOffset functions */
	glPolygonOffsetProc		glPolygonOffsetPtr;
	GLboolean			CorePolygonOffset;
	
	/* Availability of ABGR */
	GLboolean			IsABGRSupported;
	
	/* Availability of Color Matrix */
	GLboolean			IsColorMatrixSupported;
	
	/* Pointers to Convolution functions */
	glConvolutionFilter1DProc	glConvolutionFilter1DPtr;
	glConvolutionFilter2DProc	glConvolutionFilter2DPtr;
	glConvolutionParameterfProc	glConvolutionParameterfPtr;
	glConvolutionParameterfvProc	glConvolutionParameterfvPtr;
	glConvolutionParameteriProc	glConvolutionParameteriPtr;
	glConvolutionParameterivProc	glConvolutionParameterivPtr;
	glCopyConvolutionFilter1DProc	glCopyConvolutionFilter1DPtr;
	glCopyConvolutionFilter2DProc	glCopyConvolutionFilter2DPtr;
	glGetConvolutionFilterProc	glGetConvolutionFilterPtr;
	glGetConvolutionParameterfvProc	glGetConvolutionParameterfvPtr;
	glGetConvolutionParameterivProc	glGetConvolutionParameterivPtr;
	glSeparableFilter2DProc		glSeparableFilter2DPtr;
	glGetSeparableFilterProc	glGetSeparableFilterPtr;
	
	/* Availability of Convolution Border Modes */
	GLboolean			IsConvolutionBorderModesSupported;
	
	/* Pointers to Histogram functions */
	glHistogramProc			glHistogramPtr;
	glResetHistogramProc		glResetHistogramPtr;
	glGetHistogramProc		glGetHistogramPtr;
	glGetHistogramParameterfvProc	glGetHistogramParameterfvPtr;
	glGetHistogramParameterivProc	glGetHistogramParameterivPtr;
	glMinmaxProc			glMinmaxPtr;
	glResetMinmaxProc		glResetMinmaxPtr;
	glGetMinmaxProc			glGetMinmaxPtr;
	glGetMinmaxParameterfvProc	glGetMinmaxParameterfvPtr;
	glGetMinmaxParameterivProc	glGetMinmaxParameterivPtr;

	/* Pointers to Color Table functions */
	glColorTableProc		glColorTablePtr;
	glColorTableParameterivProc	glColorTableParameterivPtr;
	glColorTableParameterfvProc	glColorTableParameterfvPtr;
	glCopyColorTableProc		glCopyColorTablePtr;
	glGetColorTableProc		glGetColorTablePtr;
	glGetColorTableParameterfvProc	glGetColorTableParameterfvPtr;
	glGetColorTableParameterivProc	glGetColorTableParameterivPtr;

	/* Pointers to Color SubTable functions */
	glColorSubTableProc		glColorSubTablePtr;
	glCopyColorSubTableProc		glCopyColorSubTablePtr;
	
	/* Availability of 'Packed Pixels' */
	GLboolean			IsPackedPixelsSupported;
	
	/* Availability of 'Texture Cube Map' feature */
	GLboolean			IsTextureCubeMapSupported;
	};


/***************************************************************************/
/*Prototype of the _glaf_Verbose function (source code goes in 'contexts.c'*/
/***************************************************************************/
void	_glaf_Verbose(char *str);

#endif			/* __contexts_h__ */
