/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "public.h"


GLboolean	glaf_IsConvolutionSupported(void)
	{
	/*It returns GL_TRUE if convolution is supported, or GL_FALSE otherwise.
	
	  The use of convolution is exactly the same as the GL_ARB_imaging convolution extension, or the GL_EXT_convolution extension.
	  
	  Enumerants follow the GLAF syntax:
						GLAF_CONVOLUTION_1D			
						GLAF_CONVOLUTION_2D			
						GLAF_SEPARABLE_2D			
						GLAF_CONVOLUTION_BORDER_MODE		
						GLAF_CONVOLUTION_FILTER_SCALE		
						GLAF_CONVOLUTION_FILTER_BIAS		
						GLAF_REDUCE				
						GLAF_CONVOLUTION_FORMAT			
						GLAF_CONVOLUTION_WIDTH			
						GLAF_CONVOLUTION_HEIGHT			
						GLAF_MAX_CONVOLUTION_WIDTH		
						GLAF_MAX_CONVOLUTION_HEIGHT		
						GLAF_POST_CONVOLUTION_RED_SCALE		
						GLAF_POST_CONVOLUTION_GREEN_SCALE	
						GLAF_POST_CONVOLUTION_BLUE_SCALE	
						GLAF_POST_CONVOLUTION_ALPHA_SCALE	
						GLAF_POST_CONVOLUTION_RED_BIAS	
						GLAF_POST_CONVOLUTION_GREEN_BIAS
						GLAF_POST_CONVOLUTION_BLUE_BIAS		
						GLAF_POST_CONVOLUTION_ALPHA_BIAS	
	
	
	  GLAF functions for the convolution feature (use the same as GL_ARB_imaging or GL_EXT_convolution):
	
						glaf_ConvolutionFilter1D
						glaf_ConvolutionFilter2D
						glaf_ConvolutionParameterf
						glaf_ConvolutionParameterfv
						glaf_ConvolutionParameteri
						glaf_ConvolutionParameteriv
						glaf_CopyConvolutionFilter1D
						glaf_CopyConvolutionFilter2D
						glaf_GetConvolutionFilter
						glaf_GetConvolutionParameterfv
						glaf_GetConvolutionParameteriv
						glaf_SeparableFilter2D
						glaf_GetSeparableFilter

	
	  Note that it's very important to call glaf_IsConvolutionSupported *before* you use any of the 
	  above functions. Otherwise, if you use them and they're not supported, the application is 
	  likely to crash.
	*/
	if(GLB_current->glConvolutionFilter1DPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
void	glaf_ConvolutionFilter1D( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *image )
	{
	/*Use is the same as glConvolutionFilter1D, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glConvolutionFilter1DPtr( target, internalformat, width, format, type, image );
	}
	
	
void	glaf_ConvolutionFilter2D( GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *image )
	{
	/*Use is the same as glConvolutionFilter2D, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glConvolutionFilter2DPtr( target, internalformat, width, height, format, type, image );
	}
	

void	glaf_ConvolutionParameterf( GLenum target, GLenum pname, GLfloat params )
	{
	/*Use is the same as glConvolutionParameterf, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glConvolutionParameterfPtr( target, pname, params );
	}
	

void	glaf_ConvolutionParameterfv( GLenum target, GLenum pname, const GLfloat *params )
	{
	/*Use is the same as glConvolutionParameterfv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glConvolutionParameterfvPtr( target, pname, params );
	}
	

void	glaf_ConvolutionParameteri( GLenum target, GLenum pname, GLint params )
	{
	/*Use is the same as glConvolutionParameteri, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glConvolutionParameteriPtr( target, pname, params );
	}
	

void	glaf_ConvolutionParameteriv( GLenum target, GLenum pname, GLint *params )
	{
	/*Use is the same as glConvolutionParameteriv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glConvolutionParameterivPtr( target, pname, params );
	}
	

void	glaf_CopyConvolutionFilter1D( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width )
	{
	/*Use is the same as glCopyConvolutionFilter1D, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glCopyConvolutionFilter1DPtr( target, internalformat, x, y, width );
	}
	

void	glaf_CopyConvolutionFilter2D( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height)
	{
	/*Use is the same as glCopyConvolutionFilter2D, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glCopyConvolutionFilter2DPtr( target, internalformat, x, y, width, height);
	}
	

void	glaf_GetConvolutionFilter( GLenum target, GLenum format, GLenum type, GLvoid *image )
	{
	/*Use is the same as glGetConvolutionFilter, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetConvolutionFilterPtr( target, format, type, image );
	}
	

void	glaf_GetConvolutionParameterfv( GLenum target, GLenum pname, GLfloat *params )
	{
	/*Use is the same as glGetConvolutionParameterfv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetConvolutionParameterfvPtr( target, pname, params );
	}
	

void	glaf_GetConvolutionParameteriv( GLenum target, GLenum pname, GLint *params )
	{
	/*Use is the same as glGetConvolutionParameteriv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetConvolutionParameterivPtr( target, pname, params );
	}
	

void	glaf_SeparableFilter2D( GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *row, const GLvoid *column )
	{
	/*Use is the same as glSeparableFilter2D, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glSeparableFilter2DPtr( target, internalformat, width, height, format, type, row, column );
	}
	

void	glaf_GetSeparableFilter( GLenum target, GLenum format, GLenum type, GLvoid *row, GLvoid *column, GLvoid *span )
	{
	/*Use is the same as glGetSeparableFilter, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetSeparableFilterPtr( target, format, type, row, column, span );
	}
	

	
