/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */


#include "private.h"
#include "convolutionP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextConvolution(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>1))
		{
		/*ARB_imaging requires OpenGL 1.2 or greater*/
		if(glaf_IsOpenGLExtensionSupported("GL_ARB_imaging"))
			{
			/* ARB_imaging supported: We try to use it */
			_glaf_FillContextConvolutionARBImaging(ctx);
			if(_glaf_TestContextConvolution(ctx))
				{
				/*Success!!*/
				_glaf_Verbose("Convolution: Using GL_ARB_imaging.");
				return;
				}
			}
		}
	/*Let's see if we can use GL_EXT_convolution*/
	if(glaf_IsOpenGLExtensionSupported("GL_EXT_convolution"))
		{
		/*It seems we can use GL_EXT_convolution*/
		_glaf_FillContextConvolutionExt(ctx);
		if(_glaf_TestContextConvolution(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Convolution: Using GL_EXT_convolution.");
			return;
			}
		}
	
	/*Nope, Convolution is not supported*/
	_glaf_Verbose("Convolution: Not Supported.");
	ctx->glConvolutionFilter1DPtr=NULL;
	ctx->glConvolutionFilter2DPtr=NULL;
	ctx->glConvolutionParameterfPtr=NULL;
	ctx->glConvolutionParameterfvPtr=NULL;
	ctx->glConvolutionParameteriPtr=NULL;
	ctx->glConvolutionParameterivPtr=NULL;
	ctx->glCopyConvolutionFilter1DPtr=NULL;
	ctx->glCopyConvolutionFilter2DPtr=NULL;
	ctx->glGetConvolutionFilterPtr=NULL;
	ctx->glGetConvolutionParameterfvPtr=NULL;
	ctx->glGetConvolutionParameterivPtr=NULL;
	ctx->glSeparableFilter2DPtr=NULL;
	ctx->glGetSeparableFilterPtr=NULL;
	}

	
void	_glaf_FillContextConvolutionARBImaging(struct GLAFST_context *ctx)
	{
	ctx->glConvolutionFilter1DPtr=(glConvolutionFilter1DProc)_glaf_LookupFunction("glConvolutionFilter1D");
	ctx->glConvolutionFilter2DPtr=(glConvolutionFilter2DProc)_glaf_LookupFunction("glConvolutionFilter2D");
	ctx->glConvolutionParameterfPtr=(glConvolutionParameterfProc)_glaf_LookupFunction("glConvolutionParameterf");
	ctx->glConvolutionParameterfvPtr=(glConvolutionParameterfvProc)_glaf_LookupFunction("glConvolutionParameterfv");
	ctx->glConvolutionParameteriPtr=(glConvolutionParameteriProc)_glaf_LookupFunction("glConvolutionParameteri");
	ctx->glConvolutionParameterivPtr=(glConvolutionParameterivProc)_glaf_LookupFunction("glConvolutionParameteriv");
	ctx->glCopyConvolutionFilter1DPtr=(glCopyConvolutionFilter1DProc)_glaf_LookupFunction("glCopyConvolutionFilter1D");
	ctx->glCopyConvolutionFilter2DPtr=(glCopyConvolutionFilter2DProc)_glaf_LookupFunction("glCopyConvolutionFilter2D");
	ctx->glGetConvolutionFilterPtr=(glGetConvolutionFilterProc)_glaf_LookupFunction("glGetConvolutionFilter");
	ctx->glGetConvolutionParameterfvPtr=(glGetConvolutionParameterfvProc)_glaf_LookupFunction("glGetConvolutionParameterfv");
	ctx->glGetConvolutionParameterivPtr=(glGetConvolutionParameterivProc)_glaf_LookupFunction("glGetConvolutionParameteriv");
	ctx->glSeparableFilter2DPtr=(glSeparableFilter2DProc)_glaf_LookupFunction("glSeparableFilter2D");
	ctx->glGetSeparableFilterPtr=(glGetSeparableFilterProc)_glaf_LookupFunction("glGetSeparableFilter");
	}
	
	
void	_glaf_FillContextConvolutionExt(struct GLAFST_context *ctx)
	{
	ctx->glConvolutionFilter1DPtr=(glConvolutionFilter1DProc)_glaf_LookupFunction("glConvolutionFilter1DEXT");
	ctx->glConvolutionFilter2DPtr=(glConvolutionFilter2DProc)_glaf_LookupFunction("glConvolutionFilter2DEXT");
	ctx->glConvolutionParameterfPtr=(glConvolutionParameterfProc)_glaf_LookupFunction("glConvolutionParameterfEXT");
	ctx->glConvolutionParameterfvPtr=(glConvolutionParameterfvProc)_glaf_LookupFunction("glConvolutionParameterfvEXT");
	ctx->glConvolutionParameteriPtr=(glConvolutionParameteriProc)_glaf_LookupFunction("glConvolutionParameteriEXT");
	ctx->glConvolutionParameterivPtr=(glConvolutionParameterivProc)_glaf_LookupFunction("glConvolutionParameterivEXT");
	ctx->glCopyConvolutionFilter1DPtr=(glCopyConvolutionFilter1DProc)_glaf_LookupFunction("glCopyConvolutionFilter1DEXT");
	ctx->glCopyConvolutionFilter2DPtr=(glCopyConvolutionFilter2DProc)_glaf_LookupFunction("glCopyConvolutionFilter2DEXT");
	ctx->glGetConvolutionFilterPtr=(glGetConvolutionFilterProc)_glaf_LookupFunction("glGetConvolutionFilterEXT");
	ctx->glGetConvolutionParameterfvPtr=(glGetConvolutionParameterfvProc)_glaf_LookupFunction("glGetConvolutionParameterfvEXT");
	ctx->glGetConvolutionParameterivPtr=(glGetConvolutionParameterivProc)_glaf_LookupFunction("glGetConvolutionParameterivEXT");
	ctx->glSeparableFilter2DPtr=(glSeparableFilter2DProc)_glaf_LookupFunction("glSeparableFilter2DEXT");
	ctx->glGetSeparableFilterPtr=(glGetSeparableFilterProc)_glaf_LookupFunction("glGetSeparableFilterEXT");
	}
	
	
GLboolean	_glaf_TestContextConvolution(struct GLAFST_context *ctx)
	{
	if(ctx->glConvolutionFilter1DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glConvolutionFilter2DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glConvolutionParameterfPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glConvolutionParameterfvPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glConvolutionParameteriPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glConvolutionParameterivPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyConvolutionFilter1DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyConvolutionFilter2DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetConvolutionFilterPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetConvolutionParameterfvPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetConvolutionParameterivPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glSeparableFilter2DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetSeparableFilterPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
