/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "private.h"
#include "copy_textureP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextCopyTexture(struct GLAFST_context *ctx)
	{
	/*IMPORTANT: _glaf_FillContextSubTexture() *must* be called before _glaf_FillContextCopyTexture(), because some pointers
		    initialized by that function are directly accessed here
	*/
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>0))
		{
		/* OpenGL 1.1 or greater: We try to use OpenGL core functionality*/
		_glaf_FillContextCopyTextureCore(ctx);
		if(_glaf_TestContextCopyTexture(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Copy Texture: Supported by OpenGL Core.");
			return;
			}
		}
	/*Let's see if we can use GL_EXT_copy_texture*/
	if(glaf_IsOpenGLExtensionSupported("GL_EXT_copy_texture"))
		{
		/*It seems we can use GL_EXT_copy_texture*/
		_glaf_FillContextCopyTextureExt(ctx);
		if(_glaf_TestContextCopyTexture(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Copy Texture: Using GL_EXT_copy_texture.");
			return;
			}
		else
			{
			/*Nope, Copy Texture not supported: We will emulate it*/
			if(ctx->glTexSubImage1DPtr) /*IMPORTANT: Previously set by _glaf_FillContextSubTexture() */
				{
				/*subtexture supported -> we can *fully* emulate Copy Texture*/
				_glaf_Verbose("Copy Texture: Using emulation.");
				}
			else
				{
				/*subtexture NOT supported -> we can *partially* emulate Copy Texture*/
				_glaf_Verbose("Copy Texture: Using emulation (Copy SubTexture NOT emulated since SubTexture is not supported.");
				}
			}
		}
	ctx->glCopyTexImage1DPtr=NULL;
	ctx->glCopyTexImage2DPtr=NULL;
	ctx->glCopyTexSubImage1DPtr=NULL;
	ctx->glCopyTexSubImage2DPtr=NULL;
	}

	
void	_glaf_FillContextCopyTextureCore(struct GLAFST_context *ctx)
	{
	ctx->glCopyTexImage1DPtr=(glCopyTexImage1DProc)_glaf_LookupFunction("glCopyTexImage1D");
	ctx->glCopyTexImage2DPtr=(glCopyTexImage2DProc)_glaf_LookupFunction("glCopyTexImage2D");
	ctx->glCopyTexSubImage1DPtr=(glCopyTexSubImage1DProc)_glaf_LookupFunction("glCopyTexSubImage1D");
	ctx->glCopyTexSubImage2DPtr=(glCopyTexSubImage2DProc)_glaf_LookupFunction("glCopyTexSubImage2D");
	}
	
		
void	_glaf_FillContextCopyTextureExt(struct GLAFST_context *ctx)
	{
	ctx->glCopyTexImage1DPtr=(glCopyTexImage1DProc)_glaf_LookupFunction("glCopyTexImage1DEXT");
	ctx->glCopyTexImage2DPtr=(glCopyTexImage2DProc)_glaf_LookupFunction("glCopyTexImage2DEXT");
	ctx->glCopyTexSubImage1DPtr=(glCopyTexSubImage1DProc)_glaf_LookupFunction("glCopyTexSubImage1DEXT");
	ctx->glCopyTexSubImage2DPtr=(glCopyTexSubImage2DProc)_glaf_LookupFunction("glCopyTexSubImage2DEXT");
	}
	
		
GLboolean	_glaf_TestContextCopyTexture(struct GLAFST_context *ctx)
	{
	if(ctx->glCopyTexImage1DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyTexImage2DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyTexSubImage1DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glCopyTexSubImage2DPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	

