/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */

#ifndef __glaf_h__
#define __glaf_h__


#ifdef _WIN32
#include <windows.h>
#endif

#include <GL/gl.h>

#if defined(__cplusplus)
extern "C" {
#endif

/*********************************************************************************/
/*GLAF enumerants --don't modify these values, must match OpenGL gl.h and glext.h*/
/*********************************************************************************/

/*separate specular color*/
#define GLAF_LIGHT_MODEL_COLOR_CONTROL			0x81F8
#define GLAF_SINGLE_COLOR				0x81F9
#define GLAF_SEPARATE_SPECULAR_COLOR			0x81FA

/*texture formats*/
#define GLAF_ALPHA4					0x803B
#define GLAF_ALPHA8					0x803C
#define GLAF_ALPHA12					0x803D
#define GLAF_ALPHA16					0x803E
#define GLAF_LUMINANCE4					0x803F
#define GLAF_LUMINANCE8					0x8040
#define GLAF_LUMINANCE12				0x8041
#define GLAF_LUMINANCE16				0x8042
#define GLAF_LUMINANCE4_ALPHA4				0x8043
#define GLAF_LUMINANCE6_ALPHA2				0x8044
#define GLAF_LUMINANCE8_ALPHA8				0x8045
#define GLAF_LUMINANCE12_ALPHA4				0x8046
#define GLAF_LUMINANCE12_ALPHA12			0x8047
#define GLAF_LUMINANCE16_ALPHA16			0x8048
#define GLAF_INTENSITY					0x8049
#define GLAF_INTENSITY4					0x804A
#define GLAF_INTENSITY8					0x804B
#define GLAF_INTENSITY12				0x804C
#define GLAF_INTENSITY16				0x804D
#define GLAF_RGB4					0x804F
#define GLAF_RGB5					0x8050
#define GLAF_RGB8					0x8051
#define GLAF_RGB10					0x8052
#define GLAF_RGB12					0x8053
#define GLAF_RGB16					0x8054
#define GLAF_RGBA2					0x8055
#define GLAF_RGBA4					0x8056
#define GLAF_RGB5_A1					0x8057
#define GLAF_RGBA8					0x8058
#define GLAF_RGB10_A2					0x8059
#define GLAF_RGBA12					0x805A
#define GLAF_RGBA16					0x805B
#define GLAF_TEXTURE_RED_SIZE				0x805C
#define GLAF_TEXTURE_GREEN_SIZE				0x805D
#define GLAF_TEXTURE_BLUE_SIZE				0x805E
#define GLAF_TEXTURE_ALPHA_SIZE				0x805F
#define GLAF_TEXTURE_LUMINANCE_SIZE			0x8060
#define GLAF_TEXTURE_INTENSITY_SIZE			0x8061
#define GLAF_PROXY_TEXTURE_1D				0x8063
#define GLAF_PROXY_TEXTURE_2D				0x8064

/*texture object*/
#define GLAF_TEXTURE_PRIORITY				0x8066
#define GLAF_TEXTURE_RESIDENT				0x8067
#define GLAF_TEXTURE_BINDING_1D				0x8068
#define GLAF_TEXTURE_BINDING_2D				0x8069
#define GLAF_TEXTURE_BINDING_3D				0x806A

/*constant blend color*/
#define GLAF_CONSTANT_COLOR				0x8001
#define GLAF_ONE_MINUS_CONSTANT_COLOR			0x8002
#define GLAF_CONSTANT_ALPHA				0x8003
#define GLAF_ONE_MINUS_CONSTANT_ALPHA			0x8004
#define GLAF_BLEND_COLOR				0x8005

/*blend equations*/
#define GLAF_FUNC_ADD					0x8006
#define GLAF_MIN					0x8007
#define GLAF_MAX					0x8008
#define GLAF_BLEND_EQUATION				0x8009
#define GLAF_FUNC_SUBTRACT				0x800A
#define GLAF_FUNC_REVERSE_SUBTRACT			0x800B

/*RGBA logic-op*/
#define GLAF_COLOR_LOGIC_OP				0x0BF2

/*polygon offset*/
#define GLAF_POLYGON_OFFSET				0x8037

/*abgr*/
#define GLAF_ABGR					0x8000

/*color matrix*/
#define GLAF_COLOR					0x1800
#define GLAF_COLOR_MATRIX				0x80B1
#define GLAF_COLOR_MATRIX_STACK_DEPTH			0x80B2
#define GLAF_MAX_COLOR_MATRIX_STACK_DEPTH		0x80B3
#define GLAF_POST_COLOR_MATRIX_RED_SCALE		0x80B4
#define GLAF_POST_COLOR_MATRIX_GREEN_SCALE		0x80B5
#define GLAF_POST_COLOR_MATRIX_BLUE_SCALE		0x80B6
#define GLAF_POST_COLOR_MATRIX_ALPHA_SCALE		0x80B7
#define GLAF_POST_COLOR_MATRIX_RED_BIAS			0x80B8
#define GLAF_POST_COLOR_MATRIX_GREEN_BIAS		0x80B9
#define GLAF_POST_COLOR_MATRIX_BLUE_BIAS		0x80BA
#define GLAF_POST_COLOR_MATRIX_ALPHA_BIAS		0x80BB

/*convolution*/
#define GLAF_CONVOLUTION_1D				0x8010
#define GLAF_CONVOLUTION_2D				0x8011
#define GLAF_SEPARABLE_2D				0x8012
#define GLAF_CONVOLUTION_BORDER_MODE			0x8013
#define GLAF_CONVOLUTION_FILTER_SCALE			0x8014
#define GLAF_CONVOLUTION_FILTER_BIAS			0x8015
#define GLAF_REDUCE					0x8016
#define GLAF_CONVOLUTION_FORMAT				0x8017
#define GLAF_CONVOLUTION_WIDTH				0x8018
#define GLAF_CONVOLUTION_HEIGHT				0x8019
#define GLAF_MAX_CONVOLUTION_WIDTH			0x801A
#define GLAF_MAX_CONVOLUTION_HEIGHT			0x801B
#define GLAF_POST_CONVOLUTION_RED_SCALE			0x801C
#define GLAF_POST_CONVOLUTION_GREEN_SCALE		0x801D
#define GLAF_POST_CONVOLUTION_BLUE_SCALE		0x801E
#define GLAF_POST_CONVOLUTION_ALPHA_SCALE		0x801F
#define GLAF_POST_CONVOLUTION_RED_BIAS			0x8020
#define GLAF_POST_CONVOLUTION_GREEN_BIAS		0x8021
#define GLAF_POST_CONVOLUTION_BLUE_BIAS			0x8022
#define GLAF_POST_CONVOLUTION_ALPHA_BIAS		0x8023

/*convolution border modes*/
#define GLAF_CONSTANT_BORDER				0x8151
#define GLAF_REPLICATE_BORDER				0x8153
#define GLAF_CONVOLUTION_BORDER_COLOR			0x8154

/*histogram*/
#define GLAF_HISTOGRAM					0x8024
#define GLAF_PROXY_HISTOGRAM				0x8025
#define GLAF_HISTOGRAM_WIDTH				0x8026
#define GLAF_HISTOGRAM_FORMAT				0x8027
#define GLAF_HISTOGRAM_RED_SIZE				0x8028
#define GLAF_HISTOGRAM_GREEN_SIZE			0x8029
#define GLAF_HISTOGRAM_BLUE_SIZE			0x802A
#define GLAF_HISTOGRAM_ALPHA_SIZE			0x802B
#define GLAF_HISTOGRAM_LUMINANCE_SIZE			0x802C
#define GLAF_HISTOGRAM_SINK				0x802D
#define GLAF_MINMAX					0x802E
#define GLAF_MINMAX_FORMAT				0x802F
#define GLAF_MINMAX_SINK				0x8030
#define GLAF_TABLE_TOO_LARGE				0x8031

/*color table*/
#define GLAF_COLOR_TABLE				0x80D0
#define GLAF_POST_CONVOLUTION_COLOR_TABLE		0x80D1
#define GLAF_POST_COLOR_MATRIX_COLOR_TABLE		0x80D2
#define GLAF_PROXY_COLOR_TABLE				0x80D3
#define GLAF_PROXY_POST_CONVOLUTION_COLOR_TABLE		0x80D4
#define GLAF_PROXY_POST_COLOR_MATRIX_COLOR_TABLE	0x80D5
#define GLAF_COLOR_TABLE_SCALE				0x80D6
#define GLAF_COLOR_TABLE_BIAS				0x80D7
#define GLAF_COLOR_TABLE_FORMAT				0x80D8
#define GLAF_COLOR_TABLE_WIDTH				0x80D9
#define GLAF_COLOR_TABLE_RED_SIZE			0x80DA
#define GLAF_COLOR_TABLE_GREEN_SIZE			0x80DB
#define GLAF_COLOR_TABLE_BLUE_SIZE			0x80DC
#define GLAF_COLOR_TABLE_ALPHA_SIZE			0x80DD
#define GLAF_COLOR_TABLE_LUMINANCE_SIZE			0x80DE
#define GLAF_COLOR_TABLE_INTENSITY_SIZE			0x80DF

/*packed pixels*/
#define GLAF_UNSIGNED_BYTE_3_3_2			0x8032
#define GLAF_UNSIGNED_SHORT_4_4_4_4			0x8033
#define GLAF_UNSIGNED_SHORT_5_5_5_1			0x8034
#define GLAF_UNSIGNED_INT_8_8_8_8			0x8035
#define GLAF_UNSIGNED_INT_10_10_10_2			0x8036

/*texture cube map*/
#define GLAF_NORMAL_MAP					0x8511
#define GLAF_REFLECTION_MAP				0x8512
#define GLAF_TEXTURE_CUBE_MAP				0x8513
#define GLAF_TEXTURE_BINDING_CUBE_MAP			0x8514
#define GLAF_TEXTURE_CUBE_MAP_POSITIVE_X		0x8515
#define GLAF_TEXTURE_CUBE_MAP_NEGATIVE_X		0x8516
#define GLAF_TEXTURE_CUBE_MAP_POSITIVE_Y		0x8517
#define GLAF_TEXTURE_CUBE_MAP_NEGATIVE_Y		0x8518
#define GLAF_TEXTURE_CUBE_MAP_POSITIVE_Z		0x8519
#define GLAF_TEXTURE_CUBE_MAP_NEGATIVE_Z		0x851A
#define GLAF_PROXY_TEXTURE_CUBE_MAP			0x851B
#define GLAF_MAX_CUBE_MAP_TEXTURE_SIZE			0x851C


/****************/
/*GLAF functions*/
/****************/

/*capabilities*/
void		glaf_Version(GLint *major, GLint *minor, GLint *release);
void		glaf_OpenGLVersion(GLint *major, GLint *minor, GLint *release);
GLboolean	glaf_IsOpenGLExtensionSupported(char *name);

/*contexts*/
GLint		glaf_InitializeContext(void);
void		glaf_ReadyForContext(GLint id);
void		glaf_FreeContext(GLint id);
void		glaf_SetVerbose(void funcptr(char *));

/*state utilities*/
void		glaf_StorePixelReadState(void);
void		glaf_ResetPixelReadState(void);
void		glaf_RestorePixelReadState(void);
void		glaf_StorePixelUnpackState(void);
void		glaf_ResetPixelUnpackState(void);
void		glaf_RestorePixelUnpackState(void);
void		glaf_StorePixelPackState(void);
void		glaf_ResetPixelPackState(void);
void		glaf_RestorePixelPackState(void);

/*image utilities*/
GLint		glaf_ImageSize(GLint x, GLint y, GLenum format, GLenum type, GLint align);
GLint		glaf_FrameBufferMaxColorSize(void);
GLenum		glaf_BestUnsignedPixelType(GLint size);
GLenum		glaf_BestSignedPixelType(GLint size);
GLenum		glaf_BaseColorFormat(GLenum format);
GLint		glaf_NumComponentsFormat(GLenum format);
GLint		glaf_TypeSize(GLenum type);
GLint		glaf_ImageWidthConvolution2DIncrement(void);
GLint		glaf_ImageHeightConvolution2DIncrement(void);

/*separate specular color*/
GLboolean	glaf_IsSeparateSpecularColorSupported(void);

/*texture formats*/
GLboolean	glaf_IsTextureFormatsSupported(void);

/*texture objects*/
GLboolean	glaf_IsTextureObjectsSupported(void);
void		glaf_GenTextures(GLsizei n, GLuint *textures);
void 		glaf_DeleteTextures(GLsizei n, const GLuint *textures);
GLboolean	glaf_IsTexture(GLuint texture);
void		glaf_BindTexture(GLenum target, GLuint texture);
GLboolean	glaf_AreTexturesResident(GLsizei n, const GLuint *textures, GLboolean *residences);
void 		glaf_PrioritizeTextures(GLsizei n, const GLuint *textures, const GLclampf *priorities);

/*subtexture*/
GLboolean	glaf_IsSubTextureSupported(void);
void		glaf_TexSubImage1D( GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLenum type, const GLvoid *pixels );
void		glaf_TexSubImage2D( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *pixels );

/*copy texture*/
GLboolean	glaf_IsCopyTextureNative(void);
GLboolean	glaf_IsCopySubTextureSupported(void);
void		glaf_CopyTextureForceEmulation(int mode);
void		glaf_CopyTexImage1D( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border );
void		glaf_CopyTexImage2D( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border );
void		glaf_CopyTexSubImage1D( GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width );
void		glaf_CopyTexSubImage2D( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height );

/*blending extensions*/
GLboolean	glaf_IsBlendColorSupported(void);
void		glaf_BlendColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);
GLboolean	glaf_IsBlendMinMaxSupported(void);
GLboolean	glaf_IsBlendSubtractSupported(void);
void		glaf_BlendEquation( GLenum mode );
GLboolean	glaf_IsRGBALogicOpSupported(void);
void		glaf_EnableRGBALogicOp(void);
void		glaf_DisableRGBALogicOp(void);

/*polygon offset*/
GLboolean	glaf_IsPolygonOffsetSupported(void);
void		glaf_PolygonOffset( GLfloat factor, GLfloat units );

/*abgr*/
GLboolean	glaf_IsABGRSupported(void);

/*color matrix*/
GLboolean	glaf_IsColorMatrixSupported(void);

/*convolution*/
GLboolean	glaf_IsConvolutionSupported(void);
void		glaf_ConvolutionFilter1D( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *image );
void		glaf_ConvolutionFilter2D( GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *image );
void		glaf_ConvolutionParameterf( GLenum target, GLenum pname, GLfloat params );
void		glaf_ConvolutionParameterfv( GLenum target, GLenum pname, const GLfloat *params );
void		glaf_ConvolutionParameteri( GLenum target, GLenum pname, GLint params );
void		glaf_ConvolutionParameteriv( GLenum target, GLenum pname, GLint *params );
void		glaf_CopyConvolutionFilter1D( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width );
void		glaf_CopyConvolutionFilter2D( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height);
void		glaf_GetConvolutionFilter( GLenum target, GLenum format, GLenum type, GLvoid *image );
void		glaf_GetConvolutionParameterfv( GLenum target, GLenum pname, GLfloat *params );
void		glaf_GetConvolutionParameteriv( GLenum target, GLenum pname, GLint *params );
void		glaf_SeparableFilter2D( GLenum target, GLenum internalformat, GLsizei width, GLsizei height, GLenum format, GLenum type, const GLvoid *row, const GLvoid *column );
void		glaf_GetSeparableFilter( GLenum target, GLenum format, GLenum type, GLvoid *row, GLvoid *column, GLvoid *span );

/*convolution border modes*/
GLboolean	glaf_IsConvolutionBorderModesSupported(void);

/*histogram*/
GLboolean	glaf_IsHistogramSupported(void);
void		glaf_Histogram( GLenum target, GLsizei width, GLenum internalformat, GLboolean sink );
void		glaf_ResetHistogram( GLenum target );
void		glaf_GetHistogram( GLenum target, GLboolean reset, GLenum format, GLenum type, GLvoid *values );
void		glaf_GetHistogramParameterfv( GLenum target, GLenum pname, GLfloat *params );
void		glaf_GetHistogramParameteriv( GLenum target, GLenum pname, GLint *params );
void		glaf_Minmax( GLenum target, GLenum internalformat, GLboolean sink );
void		glaf_ResetMinmax( GLenum target );
void		glaf_GetMinmax( GLenum target, GLboolean reset, GLenum format, GLenum types, GLvoid *values );
void		glaf_GetMinmaxParameterfv( GLenum target, GLenum pname, GLfloat *params );
void		glaf_GetMinmaxParameteriv( GLenum target, GLenum pname, GLint *params );

/*color table*/
GLboolean	glaf_IsColorTableSupported(void);
void		glaf_ColorTable( GLenum target, GLenum internalformat, GLsizei width, GLenum format, GLenum type, const GLvoid *table );
void		glaf_ColorTableParameteriv(GLenum target, GLenum pname, const GLint *params);
void		glaf_ColorTableParameterfv(GLenum target, GLenum pname, const GLfloat *params);
void		glaf_CopyColorTable( GLenum target, GLenum internalformat, GLint x, GLint y, GLsizei width );
void		glaf_GetColorTable( GLenum target, GLenum format, GLenum type, GLvoid *table );
void		glaf_GetColorTableParameterfv( GLenum target, GLenum pname, GLfloat *params );
void		glaf_GetColorTableParameteriv( GLenum target, GLenum pname, GLint *params );

/*color subtable*/
GLboolean	glaf_IsColorSubTableSupported(void);
void		glaf_ColorSubTable( GLenum target, GLsizei start, GLsizei count, GLenum format, GLenum type, const GLvoid *data );
void		glaf_CopyColorSubTable( GLenum target, GLsizei start, GLint x, GLint y, GLsizei width );

/*packed pixels*/
GLboolean	glaf_IsPackedPixelsSupported(void);

/*texture cube map*/
GLboolean	glaf_IsTextureCubeMapSupported(void);

#if defined(__cplusplus)
}
#endif

#endif			/* __glaf_h__ */

