/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "public.h"


GLboolean	glaf_IsHistogramSupported(void)
	{
	/*It returns GL_TRUE if histogram is supported, or GL_FALSE otherwise.
	
	  The use of histogramming is exactly the same as the GL_ARB_imaging histogram extension, or the GL_EXT_histogram extension.
	  
	  Enumerants follow the GLAF syntax:
						GLAF_HISTOGRAM				
						GLAF_PROXY_HISTOGRAM			
						GLAF_HISTOGRAM_WIDTH			
						GLAF_HISTOGRAM_FORMAT			
						GLAF_HISTOGRAM_RED_SIZE			
						GLAF_HISTOGRAM_GREEN_SIZE		
						GLAF_HISTOGRAM_BLUE_SIZE		
						GLAF_HISTOGRAM_ALPHA_SIZE		
						GLAF_HISTOGRAM_LUMINANCE_SIZE		
						GLAF_HISTOGRAM_SINK			
						GLAF_MINMAX				
						GLAF_MINMAX_FORMAT			
						GLAF_MINMAX_SINK			
						GLAF_TABLE_TOO_LARGE			
	
	  
	  GLAF functions for the histogram feature (use the same as GL_ARB_imaging or GL_EXT_histogram):
	
						glaf_Histogram
						glaf_ResetHistogram
						glaf_GetHistogram
						glaf_GetHistogramParameterfv
						glaf_GetHistogramParameteriv
						glaf_Minmax
						glaf_ResetMinmax
						glaf_GetMinmax
						glaf_GetMinmaxParameterfv
						glaf_GetMinmaxParameteriv
	
	
	  Note that it's very important to call glaf_IsHistogramSupported *before* you use any of the 
	  above functions. Otherwise, if you use them and they're not supported, the application is 
	  likely to crash.
	*/
	if(GLB_current->glHistogramPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
void	glaf_Histogram( GLenum target, GLsizei width, GLenum internalformat, GLboolean sink )
	{
	/*Use is the same as glHistogram, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glHistogramPtr( target, width, internalformat, sink );
	}
	
	
void	glaf_ResetHistogram( GLenum target )
	{
	/*Use is the same as glResetHistogram, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glResetHistogramPtr( target );
	}
	
	
void	glaf_GetHistogram( GLenum target, GLboolean reset, GLenum format, GLenum type, GLvoid *values )
	{
	/*Use is the same as glGetHistogram, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetHistogramPtr( target, reset, format, type, values );
	}
	
	
void	glaf_GetHistogramParameterfv( GLenum target, GLenum pname, GLfloat *params )
	{
	/*Use is the same as glGetHistogramParameterfv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetHistogramParameterfvPtr( target, pname, params );
	}
	
	
void	glaf_GetHistogramParameteriv( GLenum target, GLenum pname, GLint *params )
	{
	/*Use is the same as glGetHistogramParameteriv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetHistogramParameterivPtr( target, pname, params );
	}
	
	
void	glaf_Minmax( GLenum target, GLenum internalformat, GLboolean sink )
	{
	/*Use is the same as glMinmax, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glMinmaxPtr( target, internalformat, sink );
	}
	
	
void	glaf_ResetMinmax( GLenum target )
	{
	/*Use is the same as glResetMinmax, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glResetMinmaxPtr( target );
	}
	
	
void	glaf_GetMinmax( GLenum target, GLboolean reset, GLenum format, GLenum types, GLvoid *values )
	{
	/*Use is the same as glGetMinmax, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetMinmaxPtr( target, reset, format, types, values );
	}
	
	
void	glaf_GetMinmaxParameterfv( GLenum target, GLenum pname, GLfloat *params )
	{
	/*Use is the same as glGetMinmaxParameterfv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetMinmaxParameterfvPtr( target, pname, params );
	}
	
	
void	glaf_GetMinmaxParameteriv( GLenum target, GLenum pname, GLint *params )
	{
	/*Use is the same as glGetMinmaxParameteriv, except for GLAF-supported enumerants, that must follow the GLAF naming scheme.
	*/
	GLB_current->glGetMinmaxParameterivPtr( target, pname, params );
	}
	
	
