/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "private.h"
#include "histogramP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextHistogram(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>1))
		{
		/*ARB_imaging requires OpenGL 1.2 or greater*/
		if(glaf_IsOpenGLExtensionSupported("GL_ARB_imaging"))
			{
			/* ARB_imaging supported: We try to use it */
			_glaf_FillContextHistogramARBImaging(ctx);
			if(_glaf_TestContextHistogram(ctx))
				{
				/*Success!!*/
				_glaf_Verbose("Histogram: Using GL_ARB_imaging.");
				return;
				}
			}
		}
	/*Let's see if we can use GL_EXT_histogram*/
	if(glaf_IsOpenGLExtensionSupported("GL_EXT_histogram"))
		{
		/*It seems we can use GL_EXT_histogram*/
		_glaf_FillContextHistogramExt(ctx);
		if(_glaf_TestContextHistogram(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Histogram: Using GL_EXT_histogram.");
			return;
			}
		}
	
	/*Nope, Histogram is not supported*/
	_glaf_Verbose("Histogram: Not Supported.");
	ctx->glHistogramPtr=NULL;
	ctx->glResetHistogramPtr=NULL;
	ctx->glGetHistogramPtr=NULL;
	ctx->glGetHistogramParameterfvPtr=NULL;
	ctx->glGetHistogramParameterivPtr=NULL;
	ctx->glMinmaxPtr=NULL;
	ctx->glResetMinmaxPtr=NULL;
	ctx->glGetMinmaxPtr=NULL;
	ctx->glGetMinmaxParameterfvPtr=NULL;
	ctx->glGetMinmaxParameterivPtr=NULL;
	}

	
void	_glaf_FillContextHistogramARBImaging(struct GLAFST_context *ctx)
	{
	ctx->glHistogramPtr=(glHistogramProc)_glaf_LookupFunction("glHistogram");
	ctx->glResetHistogramPtr=(glResetHistogramProc)_glaf_LookupFunction("glResetHistogram");
	ctx->glGetHistogramPtr=(glGetHistogramProc)_glaf_LookupFunction("glGetHistogram");
	ctx->glGetHistogramParameterfvPtr=(glGetHistogramParameterfvProc)_glaf_LookupFunction("glGetHistogramParameterfv");
	ctx->glGetHistogramParameterivPtr=(glGetHistogramParameterivProc)_glaf_LookupFunction("glGetHistogramParameteriv");
	ctx->glMinmaxPtr=(glMinmaxProc)_glaf_LookupFunction("glMinmax");
	ctx->glResetMinmaxPtr=(glResetMinmaxProc)_glaf_LookupFunction("glResetMinmax");
	ctx->glGetMinmaxPtr=(glGetMinmaxProc)_glaf_LookupFunction("glGetMinmax");
	ctx->glGetMinmaxParameterfvPtr=(glGetMinmaxParameterfvProc)_glaf_LookupFunction("glGetMinmaxParameterfv");
	ctx->glGetMinmaxParameterivPtr=(glGetMinmaxParameterivProc)_glaf_LookupFunction("glGetMinmaxParameteriv");
	}
	

void	_glaf_FillContextHistogramExt(struct GLAFST_context *ctx)
	{
	ctx->glHistogramPtr=(glHistogramProc)_glaf_LookupFunction("glHistogramEXT");
	ctx->glResetHistogramPtr=(glResetHistogramProc)_glaf_LookupFunction("glResetHistogramEXT");
	ctx->glGetHistogramPtr=(glGetHistogramProc)_glaf_LookupFunction("glGetHistogramEXT");
	ctx->glGetHistogramParameterfvPtr=(glGetHistogramParameterfvProc)_glaf_LookupFunction("glGetHistogramParameterfvEXT");
	ctx->glGetHistogramParameterivPtr=(glGetHistogramParameterivProc)_glaf_LookupFunction("glGetHistogramParameterivEXT");
	ctx->glMinmaxPtr=(glMinmaxProc)_glaf_LookupFunction("glMinmaxEXT");
	ctx->glResetMinmaxPtr=(glResetMinmaxProc)_glaf_LookupFunction("glResetMinmaxEXT");
	ctx->glGetMinmaxPtr=(glGetMinmaxProc)_glaf_LookupFunction("glGetMinmaxEXT");
	ctx->glGetMinmaxParameterfvPtr=(glGetMinmaxParameterfvProc)_glaf_LookupFunction("glGetMinmaxParameterfvEXT");
	ctx->glGetMinmaxParameterivPtr=(glGetMinmaxParameterivProc)_glaf_LookupFunction("glGetMinmaxParameterivEXT");
	}
	

GLboolean	_glaf_TestContextHistogram(struct GLAFST_context *ctx)
	{
	if(ctx->glHistogramPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glResetHistogramPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetHistogramPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetHistogramParameterfvPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetHistogramParameterivPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glMinmaxPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glResetMinmaxPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetMinmaxPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetMinmaxParameterfvPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glGetMinmaxParameterivPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
	
