/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "public.h"
#include <math.h>

#ifdef _WIN32
#define powf (GLfloat)pow
#endif



GLboolean	glaf_IsPolygonOffsetSupported(void)
	{
	/*It returns GL_TRUE if glaf_PolygonOffset is supported, or GL_FALSE otherwise.
	
	  DESCRIPTION of the Polygon Offset feature:
	  
	  	* First of all, check if it's supported, calling glaf_IsPolygonOffsetSupported.

		* glaf_PolygonOffset is used exactly the same as glPolygonOffset (ie, the bias is defined in depth buffer resolution
		  units, as opposed to glPolygonOffsetEXT).
	
		* To enable or disable it, call glEnable/glDisable with GLAF_POLYGON_OFFSET. 
		  Note that GLAF supports GL_FILL mode only.
		
	
	  IMPORTANT: If glaf_PolygonOffset is unsupported and you call it, the application is likely to crash.
	*/
	if(GLB_current->glPolygonOffsetPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
void	glaf_PolygonOffset( GLfloat factor, GLfloat units )
	{
	/*Use is documented at glaf_IsPolygonOffsetSupported.
	*/
	GLint		depthbits;
	GLfloat		r;
	
	if(GLB_current->CorePolygonOffset)
		{
		/*Version 1.1 or higher. Using core functionality*/
		GLB_current->glPolygonOffsetPtr(factor,units);
		}
	else
		{
		/*Version 1.0. Using extension functionality*/
		
		/* Get Z-buffer bits -not all of them are guaranteed to be effective- */
		glGetIntegerv(GL_DEPTH_BITS,&depthbits);
		
		/* We discard 4 bits -let's consider they are not effective- */
		/* note: discarding 4 bits sounds a bit exaggerated, but it's what SGI implicitly recommends at 
		   the glPolygonOffsetEXT manpage (ie: using 1/2**16 for 20bit Z-buffers and 1/2**20 for 24bit Z-buffers)*/
		depthbits-=4;
		
		/* Let's get an approximation of the Z effective unit*/
		r=1.0f/powf(2.0,depthbits);
		
		/* Conversion done */
		GLB_current->glPolygonOffsetPtr(factor,r*units);
		}
	return;
	}


