/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */


/* Headers */
#include "private.h"
#include "subtextureP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextSubTexture(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>0))
		{
		/* OpenGL 1.1 or greater: We try to use OpenGL core functionality*/
		_glaf_FillContextSubTextureCore(ctx);
		if(_glaf_TestContextSubTexture(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("SubTexture: Supported by OpenGL Core.");
			return;
			}
		}
	/*Let's see if we can use GL_EXT_subtexture*/
	if(glaf_IsOpenGLExtensionSupported("GL_EXT_subtexture"))
		{
		/*It seems we can use GL_EXT_subtexture*/
		_glaf_FillContextSubTextureExt(ctx);
		if(_glaf_TestContextSubTexture(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("SubTexture: Using GL_EXT_subtexture.");
			return;
			}
		else
			{
			/*Nope, SubTexture not supported*/
			_glaf_Verbose("SubTexture: Not Supported.");
			}
		}
	ctx->glTexSubImage1DPtr=NULL;
	ctx->glTexSubImage2DPtr=NULL;
	}

	
void	_glaf_FillContextSubTextureCore(struct GLAFST_context *ctx)
	{
	ctx->glTexSubImage1DPtr=(glTexSubImage1DProc)_glaf_LookupFunction("glTexSubImage1D");
	ctx->glTexSubImage2DPtr=(glTexSubImage2DProc)_glaf_LookupFunction("glTexSubImage2D");
	}
	
		
void	_glaf_FillContextSubTextureExt(struct GLAFST_context *ctx)
	{
	ctx->glTexSubImage1DPtr=(glTexSubImage1DProc)_glaf_LookupFunction("glTexSubImage1DEXT");
	ctx->glTexSubImage2DPtr=(glTexSubImage2DProc)_glaf_LookupFunction("glTexSubImage2DEXT");
	}
	
		
GLboolean	_glaf_TestContextSubTexture(struct GLAFST_context *ctx)
	{
	if(ctx->glTexSubImage1DPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glTexSubImage2DPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	

