/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */


/* Headers */
#include "public.h"


/********************/
/* Public Interface */
/********************/
GLboolean	glaf_IsTextureObjectsSupported(void)
	{
	/*It returns GL_TRUE if all the following GLAF enumerants and functions are supported by the system, 
	  or GL_FALSE if they're not supported:
	
	    Enumerants:
	    
		GLAF_TEXTURE_PRIORITY				
		GLAF_TEXTURE_RESIDENT				
		GLAF_TEXTURE_BINDING_1D				
		GLAF_TEXTURE_BINDING_2D				
		GLAF_TEXTURE_BINDING_3D				

		
	    Functions:
	    
		glaf_GenTextures
		glaf_DeleteTextures
		glaf_IsTexture
		glaf_BindTexture
		glaf_AreTexturesResident
		glaf_PrioritizeTextures
	
	  Note that it's very important to call this function *before* you use any of the 
	  above ones. Otherwise, if you use them and they're not supported, the application 
	  is likely to crash.
	*/
	if(GLB_current->glGenTexturesPtr==NULL) /* GLAF turns all pointers NULL when the feature is not supported, so we just need to test one */
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}


void	glaf_GenTextures(GLsizei n, GLuint *textures)
	{
	/*Use is the same as glGenTextures().
	*/
	GLB_current->glGenTexturesPtr(n,textures);
	}


void 	glaf_DeleteTextures(GLsizei n, const GLuint *textures)
	{
	/*Use is the same as glDeleteTextures().
	*/
	GLB_current->glDeleteTexturesPtr(n,textures);
	}


GLboolean	glaf_IsTexture(GLuint texture)
	{
	/*Use is the same as glIsTexture().
	*/
	return GLB_current->glIsTexturePtr(texture);
	}
	
		
void	glaf_BindTexture(GLenum target, GLuint texture)
	{
	/*Use is the same as glBindTexture().
	*/
	GLB_current->glBindTexturePtr(target,texture);
	}
	

GLboolean	glaf_AreTexturesResident(GLsizei n, const GLuint *textures, GLboolean *residences)
	{
	/*Use is the same as glAreTexturesResident().
	*/
	return GLB_current->glAreTexturesResidentPtr(n,textures,residences);
	}


void 	glaf_PrioritizeTextures(GLsizei n, const GLuint *textures, const GLclampf *priorities)
	{
	/*Use is the same as glPrioritizeTextures().
	*/
	GLB_current->glPrioritizeTexturesPtr(n,textures,priorities);
	}
	
	
	
