/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */


/* Headers */
#include "private.h"
#include "texture_objectsP.h"


/*********************/
/* Private Interface */
/*********************/
void	_glaf_FillContextTextureObjects(struct GLAFST_context *ctx)
	{
	GLint		major,minor,release;
	
	glaf_OpenGLVersion(&major,&minor,&release);
	if((major>1)||(minor>0))
		{
		/* OpenGL 1.1 or greater: We try to use OpenGL core functionality*/
		_glaf_FillContextTextureObjectsCore(ctx);
		if(_glaf_TestContextTextureObjects(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Texture Objects: Supported by OpenGL Core.");
			return;
			}
		}
	/*Let's see if we can use GL_EXT_texture_object*/
	if(glaf_IsOpenGLExtensionSupported("GL_EXT_texture_object"))
		{
		/*It seems we can use GL_EXT_texture_object*/
		_glaf_FillContextTextureObjectsExt(ctx);
		if(_glaf_TestContextTextureObjects(ctx))
			{
			/*Success!!*/
			_glaf_Verbose("Texture Objects: Using GL_EXT_texture_object.");
			return;
			}
		else
			{
			/*Nope, Texture Objects are not supported*/
			_glaf_Verbose("Texture Objects: Not Supported.");
			}
		}
	ctx->glGenTexturesPtr=NULL;
	ctx->glDeleteTexturesPtr=NULL;
	ctx->glIsTexturePtr=NULL;
	ctx->glBindTexturePtr=NULL;
	ctx->glAreTexturesResidentPtr=NULL;
	ctx->glPrioritizeTexturesPtr=NULL;
	}

	
void	_glaf_FillContextTextureObjectsCore(struct GLAFST_context *ctx)
	{
	ctx->glGenTexturesPtr=(glGenTexturesProc)_glaf_LookupFunction("glGenTextures");
	ctx->glDeleteTexturesPtr=(glDeleteTexturesProc)_glaf_LookupFunction("glDeleteTextures");
	ctx->glIsTexturePtr=(glIsTextureProc)_glaf_LookupFunction("glIsTexture");
	ctx->glBindTexturePtr=(glBindTextureProc)_glaf_LookupFunction("glBindTexture");
	ctx->glAreTexturesResidentPtr=(glAreTexturesResidentProc)_glaf_LookupFunction("glAreTexturesResident");
	ctx->glPrioritizeTexturesPtr=(glPrioritizeTexturesProc)_glaf_LookupFunction("glPrioritizeTextures");
	}
	
	
void	_glaf_FillContextTextureObjectsExt(struct GLAFST_context *ctx)
	{
	ctx->glGenTexturesPtr=(glGenTexturesProc)_glaf_LookupFunction("glGenTexturesEXT");
	ctx->glDeleteTexturesPtr=(glDeleteTexturesProc)_glaf_LookupFunction("glDeleteTexturesEXT");
	ctx->glIsTexturePtr=(glIsTextureProc)_glaf_LookupFunction("glIsTextureEXT");
	ctx->glBindTexturePtr=(glBindTextureProc)_glaf_LookupFunction("glBindTextureEXT");
	ctx->glAreTexturesResidentPtr=(glAreTexturesResidentProc)_glaf_LookupFunction("glAreTexturesResidentEXT");
	ctx->glPrioritizeTexturesPtr=(glPrioritizeTexturesProc)_glaf_LookupFunction("glPrioritizeTexturesEXT");
	}
	
	
GLboolean	_glaf_TestContextTextureObjects(struct GLAFST_context *ctx)
	{
	if(ctx->glGenTexturesPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glDeleteTexturesPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glIsTexturePtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glBindTexturePtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glAreTexturesResidentPtr==NULL)
		{
		return GL_FALSE;
		}
	if(ctx->glPrioritizeTexturesPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
